package org.specs.util

/** The ExtendedString object adds utility functions like 'uncapitalize' to Strings */
object ExtendedString {
  /** @return an ExtendedString */
  implicit def toExtendedString(s: String) = ExtendedString(s) 

  /** This class adds utility functions to Strings */
  case class ExtendedString(s: String) {
    /** @return the String s with its first character being uncapitalized: "HELLO".uncapitalize -> "hELLO" */
    def uncapitalize = s.first.toLowerCase + s.drop(1)

    /** 
     * @param remove String to suppress from the original string
     * @return a String where every occurrence of remove has been suppressed 
     */
    def removeAll(remove: String) = s.replaceAll(toReplace(remove), "")
    private def toReplace(c: String) = c.map { letter => if ("()[]{}+-\\^$|?.*".contains(letter)) ("\\" + letter) else letter }.mkString("")
    
    /**
     * Remove everything from the first occurrence of a given substring.
     */
    def removeFrom(sub: String) = if (s.indexOf(sub) >= 0) s.substring(0, s.indexOf(sub)) else s
    
    /** 
     * @param pattern regexp pattern with groups (defined using parenthesis) specifying what to search in the string s
     * @return a list with every group found
     */
    def groups(pattern: String): List[String] = {
      if (pattern == null) return List[String]()
      val matcher = java.util.regex.Pattern.compile(pattern).matcher(s)
      val groupsFound = new scala.collection.mutable.ListBuffer[String]()
      while (matcher.find) { 
        try {
         groupsFound += matcher.group(1) 
        } catch { case _ => }
      }
      groupsFound.toList
    }
    def replaceGroups(pattern: String, function: String => Any): String = {
      groups(pattern).foldLeft(s) { (res: String, g: String) => s.replace(g, function(g).toString) }
    }
    
    /** 
     * This is a shortcut for groups("(" + group + ")")
     * @param group specification of the groups to find
     * @return a list with every group found
     */
    def findAll(group: String): List[String] = groups("(" + group + ")")
    
    /**
     * @return the uncamel-ed string: MyCamelString ->  My camel string
     */
    def uncamel = {
      def uncamelChars(chars : List[Char]): List[Char] = chars match {
        case c :: rest if (Character.isUpperCase(c)) => ' ' :: Character.toLowerCase(c) :: uncamelChars(rest)
        case c :: rest => c :: uncamelChars(rest)
        case Nil => Nil
      }
      if (s.isEmpty) ""
      else List.toString(s.charAt(0) :: uncamelChars(s.substring(1).toList))
    }
  }
}
